package com.qima.gift.appcommon.account.service;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.qima.gift.appcommon.account.AccountConst;
import com.qima.gift.appcommon.account.authc.Authentication;
import com.qima.gift.appcommon.account.authc.IAccountAuthc;
import com.qima.gift.appcommon.account.bean.*;
import com.qima.gift.appcommon.account.dao.AccountAuthMapper;
import com.qima.gift.appcommon.account.dao.AccountInfoMapper;
import com.qima.gift.appcommon.account.utils.AccountUtils;
import com.qima.gift.appcommon.system.bean.AppSystemProperties;
import com.qima.gift.common.AppException;
import com.qima.gift.common.CommonConst;
import com.qima.gift.common.Result;
import com.qima.gift.common.utils.JsonUtils;
import jakarta.annotation.PostConstruct;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Map;

import static com.qima.gift.appcommon.account.AccountConst.ACCOUNT_SUPER_NAME;
import static com.qima.gift.appcommon.account.AccountConst.ACCOUNT_SUPER_USERNAME;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/4 17:48
 */
@Service
@Slf4j
public class AccountServiceImpl implements IAccountService {

    @Autowired
    private AccountAuthMapper accountAuthMapper;

    @Autowired
    private AccountInfoMapper accountInfoMapper;

    @Autowired
    private AppSystemProperties systemProperties;

    private Map<AccountConst.AccountAuthTypeEnum,IAccountAuthc> authcs = Maps.newHashMap();

    @PostConstruct
    private void init() {
        if(ACCOUNT_SUPER_USERNAME.equals(systemProperties.getSuperAdmin())) {
            AccountInfoBO bo = findAccountById(AccountConst.ACCOUNT_SUPER_ID);
            if(bo == null) {
                createAccount(ACCOUNT_SUPER_NAME,AccountConst.AccountAuthTypeEnum.UserName, ACCOUNT_SUPER_USERNAME,systemProperties.getSuperPassword());
            }
        } else {
            throw new AppException("超级管理员用户名不可更改").setM(systemProperties.getSuperAdmin());
        }
    }

    @Override
    public Result<String> doAuth(AccountConst.AccountAuthTypeEnum t, String principal, String credentials) {
        Authentication a = new Authentication();
        a.setAccountAuthTypeEnum(t);
        a.setPrincipal(principal);
        a.setCredentials(credentials);
        IAccountAuthc ia = findIAccountAuthc(t);
        Authentication r = ia.doAuth(a, null);
        if(r != null && r.isUsed()) {
            return Result.ok("认证成功", r.getAccountInfoId());
        } else {
            return Result.fail("认证失败");
        }
    }

    @Override
    public void registIAccountAuth(IAccountAuthc auth) {
        authcs.put(auth.support(),auth);
        log.info("注册认证器："+auth.getClass().getName());
    }

    protected IAccountAuthc findIAccountAuthc(AccountConst.AccountAuthTypeEnum t) {
        return authcs.get(t);
    }

    @Override
    public AccountInfoBO findAccountById(String accountId) {
        AccountInfoBO r = null;
        AccountInfo bean = accountInfoMapper.selectByPrimaryKey(accountId);
        if(ObjectUtils.allNotNull(bean)) {
            r = fillAccountInfoBO(bean);
        }
        return r;
    }

    @Override
    public AccountInfoBO findAccountByPrincipal(AccountConst.AccountAuthTypeEnum t, String p) {
        AccountInfoBO r = null;
        AccountAuth bean = findAccountAuthByPrincipal(t,p);
        if(ObjectUtils.allNotNull(bean)) {
            AccountInfo infoBean = accountInfoMapper.selectByPrimaryKey(bean.getAccountId());
            r = fillAccountInfoBO(infoBean);
        }
        return r;
    }

    @Override
    public AccountInfoBO createAccount(AccountInfo info) {
        throw new UnsupportedOperationException("暂不支持");
    }

    @Override
    public AccountInfoBO saveAccountAuth(String accountId, AccountConst.AccountAuthTypeEnum e, String p, String c) {
        AccountInfoBO info = findAccountById(accountId);
        if( info != null ) {
            AccountAuth nowAuthBean = info.findAccountAuthByType(e);
            AccountAuth authBean = findAccountAuthByPrincipal(e,p);
            if(nowAuthBean == null) {
                if(authBean != null) {
                    String m = "账户["+accountId+"]要添加的凭证与账户["+authBean.getAccountId()+"]冲突";
                    throw new AppException("账户凭证更新失败").setM(m);
                } else {
                    // 新增凭证
                    nowAuthBean = new AccountAuth();
                    AccountUtils.fillBeanBeforeInsert(nowAuthBean);
                    nowAuthBean.setAccountId(accountId);
                    nowAuthBean.setAuthType(e.getKey());
                    nowAuthBean.setPrincipal(p);
                    nowAuthBean.setCredentials(c);
                    accountAuthMapper.insert(nowAuthBean);
                    log.info("账户["+accountId+"]凭证["+nowAuthBean.getId()+"]追加成功,"+JsonUtils.toJson(nowAuthBean));
                }
            } else {
                if(authBean != null && !authBean.getId().equals(nowAuthBean.getId())) {
                    String m = "账户["+accountId+"]要添加的凭证与账户["+authBean.getAccountId()+"]冲突";
                    throw new AppException("账户凭证更新失败").setM(m);
                } else {
                    // 更新凭证
                    AccountUtils.fillBeanBeforeUpdate(nowAuthBean);
                    nowAuthBean.setPrincipal(p);
                    nowAuthBean.setCredentials(c);
                    accountAuthMapper.updateByPrimaryKey(nowAuthBean);
                    log.info("账户凭证更新成功,"+JsonUtils.toJson(nowAuthBean));
                }
            }
        } else {
            throw new AppException("账户不存在").setM("参数="+ JsonUtils.toJson(Lists.newArrayList(accountId,e,p,c)));
        }
        return findAccountById(accountId);
    }

    @Override
    public AccountInfoBO createAccount(AccountInfo info, AccountAuth auth) {
        throw new UnsupportedOperationException("暂不支持");
    }

    @Override
    public AccountInfoBO createAccount(String name, AccountConst.AccountAuthTypeEnum e, String p, String c) {
        AccountAuth authBean = findAccountAuthByPrincipal(e,p);
        if(ObjectUtils.allNotNull(authBean)) {
            String m = "账户["+authBean.getAccountId()+"]已经存在该凭证["+p+"]";
            log.warn(m);
            throw new AppException(m);
        } else {
            AccountInfo ibean = new AccountInfo();
            ibean.setName(StringUtils.isBlank(name)?"用户-"+p:name);
            AccountUtils.fillBeanBeforeInsert(ibean);
            log.info("账户["+ibean.getId()+"]生成成功");
            accountInfoMapper.insert(ibean);
            AccountAuth abean = AccountUtils.fillBeanBeforeInsert(new AccountAuth());
            abean.setAccountId(ibean.getId());
            abean.setAuthType(e.getKey());
            abean.setPrincipal(p);
            abean.setCredentials(c);
            accountAuthMapper.insert(abean);
            log.info("账户["+ibean.getId()+"]凭证["+abean.getPrincipal()+"]生成成功");
            return fillAccountInfoBO(ibean);
        }
    }

    @Override
    public AccountInfoBO doAccountState(String accountId, CommonConst.StateEnum e) {
        AccountInfo ibean = accountInfoMapper.selectByPrimaryKey(accountId);
        if(ObjectUtils.allNotNull(ibean)) {
            ibean.setState(e.getKey());
            accountInfoMapper.updateByPrimaryKey(ibean);
            return fillAccountInfoBO(ibean);
        }
        return null;
    }

    @Override
    public AccountInfoBO doAccountInfoFill(String accountId, String name, String phone) {
        AccountInfo bean = accountInfoMapper.selectByPrimaryKey(accountId);
        if(ObjectUtils.allNotNull(bean)) {
            AccountInfoBO bo = saveAccountAuth(accountId, AccountConst.AccountAuthTypeEnum.Phone, phone, phone);
            bo.getAccountInfoBean().setName(name);
            accountInfoMapper.updateByPrimaryKey(bo.getAccountInfoBean());
            return bo;
        } else {
            throw new AppException("找不到账户").setM("参数="+JsonUtils.toJson(Lists.newArrayList(accountId,name,phone)));
        }
    }

    protected AccountAuth findAccountAuthByPrincipal(AccountConst.AccountAuthTypeEnum t, String p) {
        AccountAuth r = null;
        AccountAuthExample e = new AccountAuthExample();
        e.createCriteria().andAuthTypeEqualTo(t.getKey()).andPrincipalEqualTo(p);
        List<AccountAuth> beans = accountAuthMapper.selectByExample(e);
        if(!CollectionUtils.isEmpty(beans)) {
            r = beans.stream().findFirst().get();
        }
        return r;
    }

    protected AccountInfoBO fillAccountInfoBO(AccountInfo bean) {
        List<AccountInfo> beans = ObjectUtils.allNull(bean)?null:Lists.newArrayList(bean);
        return fillAccountInfoBOs(beans).stream().findFirst().orElse(null);
    }

    protected List<AccountInfoBO> fillAccountInfoBOs(List<AccountInfo> beans) {
        Map<String,AccountInfoBO> r = Maps.newLinkedHashMap();
        if(!CollectionUtils.isEmpty(beans)) {
            // 预备
            beans.forEach(bean->{r.putIfAbsent(bean.getId(),new AccountInfoBO(bean));});
            // 查询
            AccountAuthExample e = new AccountAuthExample();
            e.createCriteria().andAccountIdIn(Lists.newArrayList(r.keySet()));
            List<AccountAuth> authBeans = accountAuthMapper.selectByExample(e);
            // 装填
            authBeans.forEach(bean->r.get(bean.getAccountId()).addAccountAuthBean(bean));
        }
        return Lists.newArrayList(r.values());
    }
}
