package com.qima.gift.mall.controller;

import cn.dev33.satoken.annotation.SaCheckRole;
import cn.dev33.satoken.stp.StpUtil;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Maps;
import com.qima.gift.appcommon.account.bean.AccountInfoBO;
import com.qima.gift.appcommon.account.service.IAccountService;
import com.qima.gift.appcommon.security.satoken.LoginDataDTO;
import com.qima.gift.appcommon.security.service.ISecurityService;
import com.qima.gift.appcommon.security.utils.SecurityUtils;
import com.qima.gift.appcommon.space.base.IFileSource;
import com.qima.gift.common.CommonConst;
import com.qima.gift.common.Result;
import com.qima.gift.common.utils.CollectionUtils;
import com.qima.gift.common.utils.JsonUtils;
import com.qima.gift.common.utils.PageUtils;
import com.qima.gift.mall.MallConst;
import com.qima.gift.mall.bean.*;
import com.qima.gift.mall.service.IMallService;
import com.qima.gift.mall.utils.MallUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.InputStreamSource;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限商户<p>
 *
 * @author King
 * @since 2023/6/1 11:09
 */
@RestController
@Tag(name = "商城")
public class MallController {

    @Autowired
    private IMallService mallService;

    @Autowired
    private ISecurityService securityService;

    @Operation(summary = "完善商户", description = "内容格式:{\"company\":\"输入的商户名\",\"name\":\"用户姓名\",\"phone\":\"获取到的手机号\"}")
    @PostMapping("/mall_merchants/fill")
    @SaCheckRole("merchant")
    public Result<MallMerchantDTO> fillMerchant(@RequestBody String json) {
        LoginDataDTO dto = securityService.getLoginData();
        Map<String,String> m = JsonUtils.toObject(json,Map.class);
        String company = m.get("company");
        String name = m.get("name");
        String phone = m.get("phone");
        MallMerchantBO bo = mallService.saveMerchant(company,name,phone,dto.getAccountId());
        if(ObjectUtils.allNull(bo)) {
            return Result.fail("完善失败");
        } else {
            return Result.ok("完善成功", MallUtils.convert(bo));
        }
    }

    @PostMapping("/mall_merchants/{merchantId}/info")
    @Operation(summary = "修改商户信息", description = "内容格式:{\"company\":\"输入的商户名\",\"name\":\"联系人姓名\",\"phone\":\"联系人手机号\"}")
    @SaCheckRole("admin")
    public Result<MallMerchantDTO> updateMerchant(@PathVariable String merchantId, @RequestBody String json) {
        Map<String,String> m = JsonUtils.toObject(json,Map.class);
        String company = m.get("company");
        String name = m.get("name");
        String phone = m.get("phone");
        MallMerchantBO bo = mallService.updateMerchant(merchantId,company,name,phone);
        if(ObjectUtils.allNull(bo)) {
            return Result.fail("修改商户信息失败");
        } else {
            return Result.ok("修改商户信息成功", MallUtils.convert(bo));
        }
    }

    @PostMapping("/mall_merchants/{merchantId}/state")
    @Operation(summary = "修改商户状态", description = "内容格式:{\"state\":\"int类型：0停用，1启用，-2删除\"}")
    @SaCheckRole("admin")
    public Result<MallMerchantDTO> updateMerchantState(@PathVariable String merchantId, @RequestBody String json) {
        Map<String,Object> m = JsonUtils.toObject(json,Map.class);
        CommonConst.StateEnum state = CommonConst.StateEnum.findEnumByKey(Integer.valueOf(m.get("state").toString()));
        MallMerchantBO bo = mallService.updateMerchantState(merchantId,state);
        if(ObjectUtils.allNull(bo)) {
            return Result.fail("修改商户状态失败");
        } else {
            return Result.ok("修改商户状态成功", MallUtils.convert(bo));
        }
    }

    @GetMapping("/mall_merchants/list")
    @Operation(summary = "商户列表", parameters = {
            @Parameter(name = "name", required = false, description = "商户模糊查询名")
    })
    @SaCheckRole("admin")
    public Result<PageInfo<MallMerchantDTO>> listMerchants(String name, Integer pageNum, Integer pageSize) {
        PageInfo<MallMerchantBO> r = mallService.findMerchants(name,pageNum,pageSize);
        if(ObjectUtils.allNotNull(r)) {
            return Result.ok("查询成功",PageUtils.convertPage(r, MallUtils::convert));
        } else {
            return Result.fail("查询失败");
        }
    }

    @GetMapping("/mall_shows/list")
    @Operation(summary = "秀列表（可指定商家）" , description = "query格式：{\"name\":\"模糊查询名\"}")
    @SaCheckRole("merchant")
    public Result<PageInfo<MallShowDTO>> mallShowList(String merchantId,@RequestParam(required = false) String query, Integer pageNum, Integer pageSize) {
        MallShowQuery q = JsonUtils.toObject(query,MallShowQuery.class);
        PageInfo<MallShowBO> r = mallService.findShowsByMerchantId(merchantId,q,pageNum,pageSize);
        if(ObjectUtils.allNotNull(r)) {
            MallMerchant merchant = mallService.findMerchantById(merchantId);
            MallMerchantMember member = mallService.findMemberById(merchant.getOwner());
            return Result.ok("查询成功",PageUtils.convertPage(r, b->{
                return MallUtils.convert(b,merchant,member);
            }));
        } else {
            return Result.fail("查询失败");
        }
    }

    @GetMapping("/mall_shows/{showId}/info")
    @Operation(summary = "秀信息")
    public Result<MallShowDTO> mallShow(@PathVariable String showId) {
        MallShowBO r = mallService.findShowById(showId);
        if(ObjectUtils.allNotNull(r)) {
            MallMerchant merchant = mallService.findMerchantById(r.getMallShow().getMerchantId());
            MallMerchantMember member = mallService.findMemberById(merchant.getOwner());
            return Result.ok("查询成功", MallUtils.convert(r, merchant, member));
        } else {
            return Result.fail("查询失败");
        }
    }

    @PostMapping("/mall_shows/{showId}/copyinfo")
    @Operation(summary = "秀复制")
    @SaCheckRole("merchant")
    public Result<MallShowDTO> copyMallShow(@PathVariable String showId) {
        MallShowBO r = mallService.doCopyShow(showId);
        if(ObjectUtils.allNotNull(r)) {
            MallMerchant merchant = mallService.findMerchantById(r.getMallShow().getMerchantId());
            MallMerchantMember member = mallService.findMemberById(merchant.getOwner());
            return Result.ok("复制成功", MallUtils.convert(r, merchant, member));
        } else {
            return Result.fail("复制失败");
        }
    }

    @PostMapping("/mall_shows/new")
    @Operation(summary = "秀创建", description = "内容格式：{\"name\":\"秀的名称\",\"des\":\"秀的描述，即分享语\"}")
    @SaCheckRole("merchant")
    public Result<MallShowDTO> createMallShow(@RequestBody String json) {
        LoginDataDTO dto = securityService.getLoginData();
        Map<String,String> m = JsonUtils.toObject(json, Map.class);
        String merchantId = dto.getAttr(MallConst.SECURITY_ATTR_MERCHANTID);
        String memberId = dto.getAttr(MallConst.SECURITY_ATTR_MEMBERID);
        String name = m.get("name");
        String des = m.get("des");
        MallShowBO r = mallService.createShow(merchantId, memberId, name, des);
        if(ObjectUtils.allNotNull(r)) {
            MallMerchant merchant = mallService.findMerchantById(r.getMallShow().getMerchantId());
            MallMerchantMember member = mallService.findMemberById(merchant.getOwner());
            return Result.ok("创建成功", MallUtils.convert(r, merchant, member));
        } else {
            return Result.fail("创建失败");
        }
    }

    @PostMapping("/mall_shows/{showId}/topic")
    @Operation(summary = "秀主题设置", description = "内容格式：{\"color\":\"主题颜色\",\"image\":\"主题图临时ID\",\"name\":\"秀的名称\",\"des\":\"秀的描述，即分享语\"}")
    @SaCheckRole("merchant")
    public Result<String> doSetShowTopic(@PathVariable String showId, @RequestBody String json) {
        Map<String,String> m = JsonUtils.toObject(json, Map.class);
        String color = m.get("color");
        String image = m.get("image");
        String name = m.get("name");
        String des = m.get("des");
        String sid = mallService.doSetShowTopic(showId,color, image, name, des);
        if(StringUtils.isNotBlank(sid)) {
            return Result.ok("秀主题设置成功", sid);
        } else {
            return Result.fail("秀主题设置失败");
        }
    }

    @PostMapping("/mall_shows/{showId}/contact")
    @Operation(summary = "秀联系人设置", description = "内容格式：{\"showContact\":\"联系人\",\"showPhone\":\"联系电话\"}")
    @SaCheckRole("merchant")
    public Result<String> doSetShowContact(@PathVariable String showId, @RequestBody String json) {
        Map<String,String> m = JsonUtils.toObject(json, Map.class);
        String showContact = m.get("showContact");
        String showPhone = m.get("showPhone");
        String sid = mallService.doSetShowContact(showId, showContact, showPhone);
        if(StringUtils.isNotBlank(sid)) {
            return Result.ok("秀联系人设置成功", sid);
        } else {
            return Result.fail("秀联系人设置失败");
        }
    }

    @PostMapping("/mall_shows/{showId}/categorys")
    @Operation(summary = "秀分类维护", description = "内容格式：[{\"id\":\"已存在的传递，不存在的不传\",\"name\":\"分类名称\"}]")
    @SaCheckRole("merchant")
    public Result<MallShowDTO> doSaveShowCategorys(@PathVariable String showId, @RequestBody String json) {
        List<MallCategoryDTO> categoryDTOSs = JsonUtils.toObjectList(json, MallCategoryDTO.class);
        List<MallCategory> categories = CollectionUtils.convert(categoryDTOSs, MallUtils::convert);
        MallShowBO r = mallService.doSaveShowCategorys(showId, categories);
        if( ObjectUtils.allNotNull(r) ) {
            MallMerchant merchant = mallService.findMerchantById(r.getMallShow().getMerchantId());
            MallMerchantMember member = mallService.findMemberById(merchant.getOwner());
            MallShowDTO tr = MallUtils.convert(r, merchant, member);
            return Result.ok("秀分类维护成功", tr);
        } else {
            return Result.fail("秀分类维护失败");
        }
    }

    @PostMapping("/mall_shows/{showId}/goods/new")
    @Operation(summary = "秀商品添加", description = "内容格式：{\"categoryId\":\"分类ID\",\"name\":\"商品名称\",\"topicImage\":\"主题图片ID\",\"desImages\":[\"描述图片ID\"],\"price\":\"20.00\",\"des\":\"分享描述\"}")
    @SaCheckRole("merchant")
    public Result<MallShowGoodsDTO> doAddShowGoods(@PathVariable String showId, @RequestBody String json) {
        MallShowGoodsDTO b = JsonUtils.toObject(json, MallShowGoodsDTO.class);
        MallGoodsSku r = mallService.doAddShowGoods(showId, b.getCategoryId(), MallUtils.convert(b));
        if( ObjectUtils.allNotNull(r) ) {
            MallShowGoodsDTO tr = MallUtils.convert(r, null);
            tr.setCategoryId(b.getCategoryId());
            tr.setCategoryName(b.getCategoryName());
            return Result.ok("秀商品添加成功", tr);
        } else {
            return Result.fail("秀商品添加失败");
        }
    }

    @PostMapping("/mall_shows/{showId}/goods/{showgoodsId}/info")
    @Operation(summary = "秀商品修改", description = "内容格式：{\"id\":\"他的ID\",\"categoryId\":\"分类ID\",\"name\":\"商品名称\",\"topicImage\":\"主题图片ID\",\"desImages\":[\"描述图片ID\"],\"price\":\"20.00\",\"des\":\"分享描述\"}")
    @SaCheckRole("merchant")
    public Result<MallShowGoodsDTO> doUpdateShowGoods(@PathVariable String showId, @PathVariable String showgoodsId, @RequestBody String json) {
        MallShowGoodsDTO b = JsonUtils.toObject(json, MallShowGoodsDTO.class);
        MallGoodsSku r = mallService.doUpdateShowGoods(showId, showgoodsId, MallUtils.convert(b));
        if( ObjectUtils.allNotNull(r) ) {
            MallShowGoodsDTO tr = MallUtils.convert(r, null);
            tr.setCategoryId(b.getCategoryId());
            return Result.ok("秀商品修改成功", tr);
        } else {
            return Result.fail("秀商品修改失败");
        }
    }

    @GetMapping("/mall_shows/{showId}/goods/{showgoodsId}/info")
    @Operation(summary = "获取秀商品信息")
    public Result<MallShowGoodsDTO> showGoodsInfo(@PathVariable String showId, @PathVariable String showgoodsId) {
        MallGoodsSku r = mallService.findShowGoods(showId, showgoodsId);
        if( ObjectUtils.allNotNull(r) ) {
            MallShowGoodsDTO dto = MallUtils.convert(r, null);
            return Result.ok("获取秀商品信息成功", dto);
        } else {
            return Result.fail("获取秀商品信息失败");
        }
    }

    @PostMapping("/mall_shows/{showId}/goods/{showgoodsId}/del")
    @Operation(summary = "秀商品删除")
    @SaCheckRole("merchant")
    public Result<String> doDelShowGoods(@PathVariable String showId, @PathVariable String showgoodsId) {
        Boolean r = mallService.doDelShowGoods(showId, showgoodsId);
        if( Boolean.TRUE.equals(r) ) {
            return Result.ok("秀商品删除成功", showgoodsId);
        } else {
            return Result.fail("秀商品删除失败");
        }
    }

    @PostMapping("/mall_shows/{showId}/publish")
    @Operation(summary = "秀发布")
    @SaCheckRole("merchant")
    public Result<String> doPublishMallShow(@PathVariable String showId) {
        Boolean r = mallService.doPublishShow(showId);
        if(Boolean.TRUE.equals(r)) {
            return Result.ok("秀发布成功", showId);
        } else {
            return Result.fail("秀发布失败");
        }
    }

    @PostMapping("/mall_shows/{showId}/del")
    @Operation(summary = "秀删除")
    @SaCheckRole("merchant")
    public Result<String> deleteShow(@PathVariable String showId) {
        Boolean r = mallService.deleteShow(showId);
        if(Boolean.TRUE.equals(r)) {
            return Result.ok("秀删除成功", showId);
        } else {
            return Result.fail("秀删除失败");
        }
    }

}
